HyperbolicAngle

Struct HyperbolicAngle 

pub struct HyperbolicAngle {
    pub angles: (f64, f64, f64),
}
Combine rotations and boosts in hyperbolic space.

“Hyperbolic rotations” describe elements of the group SO(N,1), which preserve Hyperbolics embedded in Minkowski<N+1>. Transformations include pure spatial rotations as well as “boosts”.

For two-dimensional hyperbolic surfaces, use HyperbolicAngle to implement elements of SO(2,1) which rotate points about the z-axis or boost points along the x- and y-axes. SO(2,1) transformations represent symmetries of two-dimensional hyperbolic space. The algebra so(2,1) is generated by the matrices

K_x = \begin{bmatrix} 0&0&1\\ 0&0&0\\1&0&0 \end{bmatrix} \qquad\qquad
K_y = \begin{bmatrix} 0&0&0\\ 0&0&1\\0&1&0 \end{bmatrix}\qquad\qquad
J_z = \begin{bmatrix} 0&-1&0\\ 1&0&0\\0&0&0 \end{bmatrix}

which satisfy the commutation relations

[J_z,K_x] = K_y \qquad [K_x,K_y] = -J_z \qquad [J_z, K_y] = -K_x

The group elements SO(2,1) can be obtained by exponentiating the generators near identity, i.e.,

G = \exp(aJ_z + b K_x + c K_y)

In particular,

\exp(aJ_z) = \begin{bmatrix}\cos(a)&-\sin(a)&0\\ \sin(a)&\cos(a)&0 \\ 0&0&1\end{bmatrix}
\\ \exp(bK_x) = \begin{bmatrix}\cosh(b)&0&\sinh(b)\\ 0&1&0 \\ \sinh(b)&0&\cosh(b)\end{bmatrix}
\\ \exp(cK_y) = \begin{bmatrix}1&0&0\\ 0&\cosh(c)&\sinh(c) \\ 0&\sinh(c)&\cosh(c)\end{bmatrix}

HyperbolicAngle is a struct of three real numbers $(a,b,c)$ which parameterize the generators in this exponential map. The first value $a$ may be interpreted as a rotation angle, while the latter two numbers $b, c$ may be interpreted as rapidities for boosts along the x- and y- directions, respectively. Use HyperbolicRotationMatrix to generate the matrix from the values defined by HyperbolicAngle.

Rotation about z axis:

use approxim::assert_relative_eq;
use hoomd_manifold::{
    HyperbolicAngle, HyperbolicRotate, HyperbolicRotationMatrix, Minkowski,
};
use std::f64::consts::PI;

let v = Minkowski::from([1.0, 0.0, 1.0]);
let rotation_about_z = HyperbolicAngle::from((PI / 2.0, 0.0_f64, 0.0_f64));
let matrix = HyperbolicRotationMatrix::from(rotation_about_z);
let rotated = matrix.hyperbolic_rotate(&v);
assert_relative_eq!(rotated, [0.0, 1.0, 1.0].into(), epsilon = 1e-12);

Boost in the y direction:

use approxim::assert_relative_eq;
use hoomd_manifold::{
    HyperbolicAngle, HyperbolicRotate, HyperbolicRotationMatrix, Minkowski,
};

let v = Minkowski::from([1.0, 0.0, 1.0]);
let boost_in_y = HyperbolicAngle::from((0.0_f64, 0.0_f64, 0.5_f64));
let matrix = HyperbolicRotationMatrix::from(boost_in_y);
let boosted = matrix.hyperbolic_rotate(&v);
assert_relative_eq!(
    boosted,
    [1.0, (0.5_f64).sinh(), (0.5_f64).cosh()].into(),
    epsilon = 1e-12
);

The default value HyperbolicAngle::Default returns the tuple (0.0,0.0,0.0), which corresponds to the identify transformation.

Fields

angles: (f64, f64, f64)

Get the rotation angle in radians.

Implementations

Source

impl HyperbolicAngle

pub fn to_reduced(self) -> Self

Modulo the second and third components of a HyperbolicAngle by $2 \pi$.