pub struct Minkowski<const N: usize> {
pub coordinates: [f64; N],
}Expand description
A vector in N-dimensional Minkowski space.
Minkowski<N> implements (N-1,1)-dimensional Minkowski space with the
metric signature $(+ , \cdots , + , -)$. Minkowski supports
Vector operations such as vector addition and rescaling.
Constructing Minkowski vectors
Similar to Cartesian, N-dimensional vectors can be
constructed using an array of (real-valued) coordinates. Three- and
four-dimensional vectors can also be constructed from tuples:
use hoomd_manifold::Minkowski;
fn from_array() -> Minkowski<5> {
Minkowski::from([1.0, 2.0, 3.0, 4.0, 5.0])
}
fn from_tuples() -> Minkowski<3> {
Minkowski::from((6.0, 7.0, 8.0))
}Operating on Minkowski vectors
Minkowski implements everything from Vector, which includes vector
addition/subtraction and multiplication by a scalar.
use hoomd_manifold::Minkowski;
// Vector addition
let mut a = Minkowski::from([1.0, 1.0, 1.0, 1.0]);
let mut b = Minkowski::from([0.0, 0.0, 0.0, 2.0]);
a += b;
// Multiplication by a scalar
let mut c = a * 4.0;
// Division by a scalar
c /= 2.0;
assert_eq!(c, [2.0, 2.0, 2.0, 6.0].into());The distance metric on Minkowski space is given by the “spacetime interval”
d_M^2(\vec{u},\vec{v}) = (\vec{u}-\vec{v})^T \eta (\vec{u}-\vec{v})
= (u_1-v_1)^2 +\cdots + (u_{N-1}-v_{N-1})^2 - (u_N - v_N)^2Note that because this metric is not positive-definite, Minkowski this
“spacetime interval” is not a true inner-product, and therefore
Minkowski does not implement the methods of InnerProduct.
use hoomd_manifold::Minkowski;
use hoomd_vector::Metric;
let x = Minkowski::from([1.0, 0.0, 5.0]);
let y = Minkowski::from([0.0, 0.0, 3.0]);
assert_eq!(-3.0, x.distance_squared(&y));Fields
coordinates: [f64; N]The vector’s coordinates.
The final component is the one associated with a minus sign (-) in the metric.