Spherical

Struct Spherical 

pub struct Spherical<const N: usize> { /* private fields */ }
Point on the surface of a sphere.

Spherical is a point on a unit N-sphere embedded in (N+1)-dimensional euclidean space. Explicitly, the N-sphere is defined by the set of (N+1)-dimensional points whose components satisfy

x_1^2 + x_2^2 + \cdots + x_{N+1}^1 = 1.0

Note that the radius is fixed to be 1.0.

Implementations

Source

impl<const N: usize> Spherical<N>

pub fn coordinates(&self) -> &[f64; N]

Get the coordinates of the point

pub fn point(&self) -> &Cartesian<N>

Get the point of the sphere

pub fn from_cartesian_coordinates(point: Cartesian<N>) -> Spherical<N>

Construct a Sphere given a Cartesian vector and a radius.

Panics

Panics when the point is not sufficiently close to the sphere’s surface.

pub fn stereographic_projection(&self) -> Vec<f64>

Implements a stereographic projection from the N-sphere to an N-dimensional plane.

Example
use hoomd_manifold::Spherical;
use hoomd_vector::Cartesian;

let x = Cartesian::from([0.5_f64.sqrt(), 0.0, -(0.5_f64.sqrt())]);
let projection =
    Spherical::from_cartesian_coordinates(x).stereographic_projection();
assert_eq!(
    [1.0 / (2.0_f64.sqrt() + 1.0), 0.0],
    [projection[0], projection[1]]
);
Source

impl Spherical<3>

pub fn from_polar_coordinates(theta: f64, phi: f64) -> Spherical<3>

Create a 2-sphere from spherical coordinates

Source

impl Spherical<4>

pub fn from_polar_coordinates( theta: f64, phi_1: f64, phi_2: f64, ) -> Spherical<4>

Create a 3-sphere from spherical coordinates